// Hopfield Neural Network
// Written by David J. Stein, Esq. See "License.html" for terms and conditions of use.
//
// This module contains the code for the front-end form used to run the Hopfield neural network application.

#region Using declarations

using System;
using System.Xml;
using System.Xml.Schema;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

#endregion

namespace Hopfield_Network_NS {

	public class HopfieldNetworkApplication : System.Windows.Forms.Form {

		#region Variables, Initialization Code, and Main

		private System.Windows.Forms.GroupBox groupBox1;
		private System.Windows.Forms.GroupBox groupBox2;
		private System.Windows.Forms.GroupBox groupBox3;
		private System.Windows.Forms.GroupBox groupBox4;
		private System.Windows.Forms.PictureBox picNetwork;
		private System.Windows.Forms.PictureBox picInputPattern;
		private System.Windows.Forms.Button btnInitializeNetwork;
		public System.Windows.Forms.PictureBox picOutputPattern;
		private System.Windows.Forms.Button btnAddPattern;
		private System.Windows.Forms.NumericUpDown nudNetworkWidth;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.NumericUpDown nudNetworkHeight;
		private System.Windows.Forms.Button btnSaveNetwork;
		private System.Windows.Forms.TextBox txtName;
		private System.Windows.Forms.Label label4;
		private System.Windows.Forms.Button btnLoadNetwork;
		private System.Windows.Forms.Button btnClearPattern;
		private System.Windows.Forms.Button btnRandomizePattern;
		private System.Windows.Forms.Button btnSavePattern;
		private System.Windows.Forms.Button btnLoadPattern;
		private System.ComponentModel.IContainer components;
		private System.Windows.Forms.Timer timerRedrawNetwork;
		private System.Windows.Forms.Button btnClearOutputPattern;
		private System.Windows.Forms.Button btnAnalyzePatternStep;
		private System.Windows.Forms.Button btnAnalyzePatternComplete;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.TextBox txtTemplates;
		private System.Windows.Forms.GroupBox groupBox5;
		private System.Windows.Forms.GroupBox groupBox6;
		private System.Windows.Forms.Button btnAnalyze;

		private HopfieldNetwork network;
		private int iMouseX = 0, iMouseY = 0;
		private string XmlLoadErrors = "";
		private int iSelectedNeuron = 0;
		private int iLastClicked = 0;
		private int iLastMoved = 0;

		public HopfieldNetworkApplication() {
			InitializeComponent();
			network = new HopfieldNetwork(10, 10);
			iLastMoved = System.DateTime.Now.Hour * 3600 + System.DateTime.Now.Minute * 60 + System.DateTime.Now.Second;
		}

		[STAThread]
		static void Main()  {
			Application.Run(new HopfieldNetworkApplication());
		}

		#endregion

		#region Windows Form Designer generated code

		protected override void Dispose(bool disposing) {
			if (disposing) {
				if (components != null) {
					components.Dispose();
				}
			}
			base.Dispose(disposing);
		}

		private void InitializeComponent() {
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(HopfieldNetworkApplication));
			this.picNetwork = new System.Windows.Forms.PictureBox();
			this.picInputPattern = new System.Windows.Forms.PictureBox();
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.btnLoadPattern = new System.Windows.Forms.Button();
			this.btnSavePattern = new System.Windows.Forms.Button();
			this.btnRandomizePattern = new System.Windows.Forms.Button();
			this.btnClearPattern = new System.Windows.Forms.Button();
			this.btnInitializeNetwork = new System.Windows.Forms.Button();
			this.groupBox2 = new System.Windows.Forms.GroupBox();
			this.groupBox3 = new System.Windows.Forms.GroupBox();
			this.picOutputPattern = new System.Windows.Forms.PictureBox();
			this.groupBox4 = new System.Windows.Forms.GroupBox();
			this.btnAnalyze = new System.Windows.Forms.Button();
			this.label2 = new System.Windows.Forms.Label();
			this.txtTemplates = new System.Windows.Forms.TextBox();
			this.btnLoadNetwork = new System.Windows.Forms.Button();
			this.label4 = new System.Windows.Forms.Label();
			this.txtName = new System.Windows.Forms.TextBox();
			this.btnSaveNetwork = new System.Windows.Forms.Button();
			this.label3 = new System.Windows.Forms.Label();
			this.nudNetworkHeight = new System.Windows.Forms.NumericUpDown();
			this.label1 = new System.Windows.Forms.Label();
			this.nudNetworkWidth = new System.Windows.Forms.NumericUpDown();
			this.btnAddPattern = new System.Windows.Forms.Button();
			this.btnClearOutputPattern = new System.Windows.Forms.Button();
			this.btnAnalyzePatternComplete = new System.Windows.Forms.Button();
			this.btnAnalyzePatternStep = new System.Windows.Forms.Button();
			this.timerRedrawNetwork = new System.Windows.Forms.Timer(this.components);
			this.groupBox5 = new System.Windows.Forms.GroupBox();
			this.groupBox6 = new System.Windows.Forms.GroupBox();
			this.groupBox1.SuspendLayout();
			this.groupBox2.SuspendLayout();
			this.groupBox3.SuspendLayout();
			this.groupBox4.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.nudNetworkHeight)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.nudNetworkWidth)).BeginInit();
			this.groupBox5.SuspendLayout();
			this.groupBox6.SuspendLayout();
			this.SuspendLayout();
			// 
			// picNetwork
			// 
			this.picNetwork.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.picNetwork.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.picNetwork.Location = new System.Drawing.Point(8, 16);
			this.picNetwork.Name = "picNetwork";
			this.picNetwork.Size = new System.Drawing.Size(448, 448);
			this.picNetwork.TabIndex = 0;
			this.picNetwork.TabStop = false;
			this.picNetwork.Click += new System.EventHandler(this.picNetwork_Click);
			this.picNetwork.Resize += new System.EventHandler(this.picNetwork_Resize);
			this.picNetwork.Paint += new System.Windows.Forms.PaintEventHandler(this.picNetwork_Paint);
			this.picNetwork.MouseMove += new System.Windows.Forms.MouseEventHandler(this.picNetwork_MouseMove);
			// 
			// picInputPattern
			// 
			this.picInputPattern.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.picInputPattern.Location = new System.Drawing.Point(8, 16);
			this.picInputPattern.Name = "picInputPattern";
			this.picInputPattern.Size = new System.Drawing.Size(208, 208);
			this.picInputPattern.TabIndex = 1;
			this.picInputPattern.TabStop = false;
			this.picInputPattern.Click += new System.EventHandler(this.picInputPattern_Click);
			this.picInputPattern.Paint += new System.Windows.Forms.PaintEventHandler(this.picInputPattern_Paint);
			this.picInputPattern.MouseMove += new System.Windows.Forms.MouseEventHandler(this.picInputPattern_MouseMove);
			// 
			// groupBox1
			// 
			this.groupBox1.Controls.Add(this.picInputPattern);
			this.groupBox1.Location = new System.Drawing.Point(176, 8);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(224, 232);
			this.groupBox1.TabIndex = 2;
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = "Input Pattern";
			// 
			// btnLoadPattern
			// 
			this.btnLoadPattern.Location = new System.Drawing.Point(80, 48);
			this.btnLoadPattern.Name = "btnLoadPattern";
			this.btnLoadPattern.Size = new System.Drawing.Size(72, 23);
			this.btnLoadPattern.TabIndex = 5;
			this.btnLoadPattern.Text = "&Load";
			this.btnLoadPattern.Click += new System.EventHandler(this.btnLoadPattern_Click);
			// 
			// btnSavePattern
			// 
			this.btnSavePattern.Location = new System.Drawing.Point(8, 48);
			this.btnSavePattern.Name = "btnSavePattern";
			this.btnSavePattern.Size = new System.Drawing.Size(72, 23);
			this.btnSavePattern.TabIndex = 4;
			this.btnSavePattern.Text = "Save";
			this.btnSavePattern.Click += new System.EventHandler(this.btnSavePattern_Click);
			// 
			// btnRandomizePattern
			// 
			this.btnRandomizePattern.Location = new System.Drawing.Point(80, 24);
			this.btnRandomizePattern.Name = "btnRandomizePattern";
			this.btnRandomizePattern.Size = new System.Drawing.Size(72, 23);
			this.btnRandomizePattern.TabIndex = 3;
			this.btnRandomizePattern.Text = "&Randomize";
			this.btnRandomizePattern.Click += new System.EventHandler(this.btnRandomizePattern_Click);
			// 
			// btnClearPattern
			// 
			this.btnClearPattern.Location = new System.Drawing.Point(8, 24);
			this.btnClearPattern.Name = "btnClearPattern";
			this.btnClearPattern.Size = new System.Drawing.Size(72, 23);
			this.btnClearPattern.TabIndex = 2;
			this.btnClearPattern.Text = "&Clear";
			this.btnClearPattern.Click += new System.EventHandler(this.btnClearPattern_Click);
			// 
			// btnInitializeNetwork
			// 
			this.btnInitializeNetwork.Location = new System.Drawing.Point(8, 176);
			this.btnInitializeNetwork.Name = "btnInitializeNetwork";
			this.btnInitializeNetwork.Size = new System.Drawing.Size(72, 23);
			this.btnInitializeNetwork.TabIndex = 3;
			this.btnInitializeNetwork.Text = "Clear";
			this.btnInitializeNetwork.Click += new System.EventHandler(this.btnInitializeNetwork_Click);
			// 
			// groupBox2
			// 
			this.groupBox2.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.groupBox2.Controls.Add(this.picNetwork);
			this.groupBox2.Location = new System.Drawing.Point(408, 8);
			this.groupBox2.Name = "groupBox2";
			this.groupBox2.Size = new System.Drawing.Size(464, 472);
			this.groupBox2.TabIndex = 4;
			this.groupBox2.TabStop = false;
			this.groupBox2.Text = "Network";
			// 
			// groupBox3
			// 
			this.groupBox3.Controls.Add(this.picOutputPattern);
			this.groupBox3.Location = new System.Drawing.Point(176, 248);
			this.groupBox3.Name = "groupBox3";
			this.groupBox3.Size = new System.Drawing.Size(224, 232);
			this.groupBox3.TabIndex = 5;
			this.groupBox3.TabStop = false;
			this.groupBox3.Text = "Output Pattern";
			// 
			// picOutputPattern
			// 
			this.picOutputPattern.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.picOutputPattern.Location = new System.Drawing.Point(8, 16);
			this.picOutputPattern.Name = "picOutputPattern";
			this.picOutputPattern.Size = new System.Drawing.Size(208, 208);
			this.picOutputPattern.TabIndex = 2;
			this.picOutputPattern.TabStop = false;
			this.picOutputPattern.Paint += new System.Windows.Forms.PaintEventHandler(this.picOutputPattern_Paint);
			// 
			// groupBox4
			// 
			this.groupBox4.Controls.Add(this.btnAnalyze);
			this.groupBox4.Controls.Add(this.label2);
			this.groupBox4.Controls.Add(this.txtTemplates);
			this.groupBox4.Controls.Add(this.btnLoadNetwork);
			this.groupBox4.Controls.Add(this.label4);
			this.groupBox4.Controls.Add(this.txtName);
			this.groupBox4.Controls.Add(this.btnSaveNetwork);
			this.groupBox4.Controls.Add(this.label3);
			this.groupBox4.Controls.Add(this.nudNetworkHeight);
			this.groupBox4.Controls.Add(this.label1);
			this.groupBox4.Controls.Add(this.nudNetworkWidth);
			this.groupBox4.Controls.Add(this.btnInitializeNetwork);
			this.groupBox4.Location = new System.Drawing.Point(8, 8);
			this.groupBox4.Name = "groupBox4";
			this.groupBox4.Size = new System.Drawing.Size(160, 232);
			this.groupBox4.TabIndex = 6;
			this.groupBox4.TabStop = false;
			this.groupBox4.Text = "Configure Network";
			// 
			// btnAnalyze
			// 
			this.btnAnalyze.Location = new System.Drawing.Point(80, 176);
			this.btnAnalyze.Name = "btnAnalyze";
			this.btnAnalyze.Size = new System.Drawing.Size(72, 23);
			this.btnAnalyze.TabIndex = 14;
			this.btnAnalyze.Text = "Analyze";
			this.btnAnalyze.Click += new System.EventHandler(this.btnAnalyze_Click);
			// 
			// label2
			// 
			this.label2.Location = new System.Drawing.Point(8, 72);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(64, 23);
			this.label2.TabIndex = 13;
			this.label2.Text = "Templates";
			this.label2.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// txtTemplates
			// 
			this.txtTemplates.Location = new System.Drawing.Point(72, 72);
			this.txtTemplates.Name = "txtTemplates";
			this.txtTemplates.ReadOnly = true;
			this.txtTemplates.Size = new System.Drawing.Size(80, 20);
			this.txtTemplates.TabIndex = 12;
			this.txtTemplates.Text = "0";
			// 
			// btnLoadNetwork
			// 
			this.btnLoadNetwork.Location = new System.Drawing.Point(80, 200);
			this.btnLoadNetwork.Name = "btnLoadNetwork";
			this.btnLoadNetwork.Size = new System.Drawing.Size(72, 23);
			this.btnLoadNetwork.TabIndex = 11;
			this.btnLoadNetwork.Text = "Load";
			this.btnLoadNetwork.Click += new System.EventHandler(this.btnLoadNetwork_Click);
			// 
			// label4
			// 
			this.label4.Location = new System.Drawing.Point(56, 16);
			this.label4.Name = "label4";
			this.label4.Size = new System.Drawing.Size(40, 23);
			this.label4.TabIndex = 10;
			this.label4.Text = "Name";
			this.label4.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtName
			// 
			this.txtName.Location = new System.Drawing.Point(8, 40);
			this.txtName.Name = "txtName";
			this.txtName.Size = new System.Drawing.Size(144, 20);
			this.txtName.TabIndex = 9;
			this.txtName.Text = "Sample Network";
			this.txtName.TextChanged += new System.EventHandler(this.txtName_TextChanged);
			// 
			// btnSaveNetwork
			// 
			this.btnSaveNetwork.Location = new System.Drawing.Point(8, 200);
			this.btnSaveNetwork.Name = "btnSaveNetwork";
			this.btnSaveNetwork.Size = new System.Drawing.Size(72, 23);
			this.btnSaveNetwork.TabIndex = 8;
			this.btnSaveNetwork.Text = "Save";
			this.btnSaveNetwork.Click += new System.EventHandler(this.btnSaveNetwork_Click);
			// 
			// label3
			// 
			this.label3.Location = new System.Drawing.Point(8, 136);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(48, 23);
			this.label3.TabIndex = 7;
			this.label3.Text = "Height";
			this.label3.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// nudNetworkHeight
			// 
			this.nudNetworkHeight.Location = new System.Drawing.Point(72, 136);
			this.nudNetworkHeight.Minimum = new System.Decimal(new int[] {
																								 1,
																								 0,
																								 0,
																								 0});
			this.nudNetworkHeight.Name = "nudNetworkHeight";
			this.nudNetworkHeight.Size = new System.Drawing.Size(80, 20);
			this.nudNetworkHeight.TabIndex = 6;
			this.nudNetworkHeight.Value = new System.Decimal(new int[] {
																							  10,
																							  0,
																							  0,
																							  0});
			this.nudNetworkHeight.ValueChanged += new System.EventHandler(this.nudNetworkHeight_ValueChanged);
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(8, 104);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(40, 23);
			this.label1.TabIndex = 5;
			this.label1.Text = "Width";
			this.label1.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// nudNetworkWidth
			// 
			this.nudNetworkWidth.Location = new System.Drawing.Point(72, 104);
			this.nudNetworkWidth.Minimum = new System.Decimal(new int[] {
																								1,
																								0,
																								0,
																								0});
			this.nudNetworkWidth.Name = "nudNetworkWidth";
			this.nudNetworkWidth.Size = new System.Drawing.Size(80, 20);
			this.nudNetworkWidth.TabIndex = 4;
			this.nudNetworkWidth.Value = new System.Decimal(new int[] {
																							 10,
																							 0,
																							 0,
																							 0});
			this.nudNetworkWidth.ValueChanged += new System.EventHandler(this.nudNetworkWidth_ValueChanged);
			// 
			// btnAddPattern
			// 
			this.btnAddPattern.Location = new System.Drawing.Point(8, 24);
			this.btnAddPattern.Name = "btnAddPattern";
			this.btnAddPattern.Size = new System.Drawing.Size(144, 23);
			this.btnAddPattern.TabIndex = 4;
			this.btnAddPattern.Text = "&Add Pattern as Template";
			this.btnAddPattern.Click += new System.EventHandler(this.btnAddPattern_Click);
			// 
			// btnClearOutputPattern
			// 
			this.btnClearOutputPattern.Location = new System.Drawing.Point(8, 56);
			this.btnClearOutputPattern.Name = "btnClearOutputPattern";
			this.btnClearOutputPattern.Size = new System.Drawing.Size(144, 23);
			this.btnClearOutputPattern.TabIndex = 9;
			this.btnClearOutputPattern.Text = "&Initialize Output Pattern";
			this.btnClearOutputPattern.Click += new System.EventHandler(this.btnClearOutputPattern_Click);
			// 
			// btnAnalyzePatternComplete
			// 
			this.btnAnalyzePatternComplete.Location = new System.Drawing.Point(8, 120);
			this.btnAnalyzePatternComplete.Name = "btnAnalyzePatternComplete";
			this.btnAnalyzePatternComplete.Size = new System.Drawing.Size(144, 23);
			this.btnAnalyzePatternComplete.TabIndex = 5;
			this.btnAnalyzePatternComplete.Text = "Analyze Pattern (&Full)";
			this.btnAnalyzePatternComplete.Click += new System.EventHandler(this.btnAnalyzePatternComplete_Click);
			// 
			// btnAnalyzePatternStep
			// 
			this.btnAnalyzePatternStep.Location = new System.Drawing.Point(8, 88);
			this.btnAnalyzePatternStep.Name = "btnAnalyzePatternStep";
			this.btnAnalyzePatternStep.Size = new System.Drawing.Size(144, 23);
			this.btnAnalyzePatternStep.TabIndex = 8;
			this.btnAnalyzePatternStep.Text = "Analyze Pattern (&Step)";
			this.btnAnalyzePatternStep.Click += new System.EventHandler(this.btnAnalyzePatternStep_Click);
			// 
			// timerRedrawNetwork
			// 
			this.timerRedrawNetwork.Enabled = true;
			this.timerRedrawNetwork.Interval = 1000;
			this.timerRedrawNetwork.Tick += new System.EventHandler(this.timerRedrawNetwork_Tick);
			// 
			// groupBox5
			// 
			this.groupBox5.Controls.Add(this.btnSavePattern);
			this.groupBox5.Controls.Add(this.btnLoadPattern);
			this.groupBox5.Controls.Add(this.btnClearPattern);
			this.groupBox5.Controls.Add(this.btnRandomizePattern);
			this.groupBox5.Location = new System.Drawing.Point(8, 248);
			this.groupBox5.Name = "groupBox5";
			this.groupBox5.Size = new System.Drawing.Size(160, 80);
			this.groupBox5.TabIndex = 7;
			this.groupBox5.TabStop = false;
			this.groupBox5.Text = "Configure Input Pattern";
			// 
			// groupBox6
			// 
			this.groupBox6.Controls.Add(this.btnClearOutputPattern);
			this.groupBox6.Controls.Add(this.btnAnalyzePatternComplete);
			this.groupBox6.Controls.Add(this.btnAnalyzePatternStep);
			this.groupBox6.Controls.Add(this.btnAddPattern);
			this.groupBox6.Location = new System.Drawing.Point(8, 328);
			this.groupBox6.Name = "groupBox6";
			this.groupBox6.Size = new System.Drawing.Size(160, 152);
			this.groupBox6.TabIndex = 8;
			this.groupBox6.TabStop = false;
			this.groupBox6.Text = "Use Network";
			// 
			// HopfieldNetworkApplication
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(880, 486);
			this.Controls.Add(this.groupBox6);
			this.Controls.Add(this.groupBox5);
			this.Controls.Add(this.groupBox4);
			this.Controls.Add(this.groupBox3);
			this.Controls.Add(this.groupBox2);
			this.Controls.Add(this.groupBox1);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Name = "HopfieldNetworkApplication";
			this.Text = "Hopfield Neural Network";
			this.groupBox1.ResumeLayout(false);
			this.groupBox2.ResumeLayout(false);
			this.groupBox3.ResumeLayout(false);
			this.groupBox4.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.nudNetworkHeight)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.nudNetworkWidth)).EndInit();
			this.groupBox5.ResumeLayout(false);
			this.groupBox6.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		#region User Interface Event Handlers (Button Clicks, Textboxes, Etc.)

		private void btnClearOutputPattern_Click(object sender, System.EventArgs e) {
			for (int i = 0; i < network.aNeurons.Count; i++)
				((Neuron) network.aNeurons[i]).x = network.bInputPattern[i];
			picOutputPattern.Refresh();
		}

		private void btnAnalyzePatternStep_Click(object sender, System.EventArgs e) {
			ActivateButtons(false);
			network.Analyze(this);
			ActivateButtons(true);
		}

		private void btnAnalyzePatternComplete_Click(object sender, System.EventArgs e) {
			int count = 0;
			ActivateButtons(false);
			while (count < 1000) {
				picOutputPattern.Refresh();
				if (network.Analyze(this) == 0)
					break;
				count++;
			}
			ActivateButtons(true);
		}

		private void txtName_TextChanged(object sender, System.EventArgs e) {
			network.strName = txtName.Text;
		}

		private void picNetwork_Click(object sender, System.EventArgs e) {
			int x = network.iWidth * (iMouseX - 10) / (picNetwork.Width - 25);
			int y = network.iHeight * (iMouseY - 10) / (picNetwork.Height - 25);
			if ((x >= 0) && (x < network.iWidth) && (y >= 0) && (y < network.iHeight)) {  // clicked within network map
				iSelectedNeuron = y * network.iWidth + x;
				iLastClicked = System.DateTime.Now.Hour * 3600 + System.DateTime.Now.Minute * 60 + System.DateTime.Now.Second;
				picNetwork.Refresh();
			}
		}

		private void btnClearPattern_Click(object sender, System.EventArgs e) {
			for (int i = 0; i < network.bInputPattern.Length; i++)
				network.bInputPattern[i] = false;
			picInputPattern.Refresh();
		}

		private void btnInitializeNetwork_Click(object sender, System.EventArgs e) {
			foreach (object synapseobject in network.aSynapses)
				((Synapse) synapseobject).weight = 0;
			network.iTemplates = 0;
			txtTemplates.Text = "0";
			picNetwork.Refresh();
		}

		private void btnAddPattern_Click(object sender, System.EventArgs e) {
			network.AddTemplate();
			txtTemplates.Text = network.iTemplates.ToString();
			picOutputPattern.Refresh();
			picNetwork.Refresh();
		}

		private void nudNetworkWidth_ValueChanged(object sender, System.EventArgs e) {
			network.CreateNetwork((int) nudNetworkWidth.Value, (int) nudNetworkHeight.Value);
			picInputPattern.Refresh();
			picOutputPattern.Refresh();
			picNetwork.Refresh();
		}

		private void nudNetworkHeight_ValueChanged(object sender, System.EventArgs e) {
			network.CreateNetwork((int) nudNetworkWidth.Value, (int) nudNetworkHeight.Value);
			picInputPattern.Refresh();
			picOutputPattern.Refresh();
			picNetwork.Refresh();
		}

		private void btnRandomizePattern_Click(object sender, System.EventArgs e) {
			System.Random r = new System.Random();
			for (int i = 0; i < network.iWidth * network.iHeight; i++) {
				network.bInputPattern[i] = ((Neuron) network.aNeurons[i]).x = (r.Next(2) == 0);
			}
			picInputPattern.Refresh();
			picOutputPattern.Refresh();
		}

		private void picInputPattern_Click(object sender, System.EventArgs e) {
			int x = network.iWidth * (iMouseX - 10) / (picInputPattern.Width - 25);
			int y = network.iHeight * (iMouseY - 10) / (picInputPattern.Height - 25);
			if ((x >= 0) && (x < network.iWidth) && (y >= 0) && (y < network.iHeight)) {  // clicked in rectangle
				network.bInputPattern[y * network.iWidth + x] = !network.bInputPattern[y * network.iWidth + x];
				picInputPattern.Refresh();
			}
		}

		private void btnAnalyze_Click(object sender, System.EventArgs e) {
			iSelectedNeuron = -1;
			iLastClicked = System.DateTime.Now.Hour * 3600 + System.DateTime.Now.Minute * 60 + System.DateTime.Now.Second;
			picNetwork.Refresh();
		}

		#endregion
		
		#region Automated Event Handlers

		private void picNetwork_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e) {
			iMouseX= e.X;
			iMouseY = e.Y;
		}

		private void picInputPattern_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e) {
			iMouseX = e.X;
			iMouseY = e.Y;
		}

		private void timerRedrawNetwork_Tick(object sender, System.EventArgs e) {
			picNetwork.Refresh();
		}

		private void picNetwork_Resize(object sender, System.EventArgs e) {
			Refresh();
		}

		private void ActivateButtons(bool activate) {
			txtName.Enabled =
				btnAddPattern.Enabled =
				btnAnalyze.Enabled =
				btnAnalyzePatternComplete.Enabled =
				btnAnalyzePatternStep.Enabled =
				btnClearOutputPattern.Enabled =
				btnClearPattern.Enabled =
				btnInitializeNetwork.Enabled =
				btnLoadNetwork.Enabled =
				btnLoadPattern.Enabled =
				btnRandomizePattern.Enabled =
				btnSaveNetwork.Enabled =
				btnSavePattern.Enabled =
				nudNetworkHeight.Enabled =
				nudNetworkWidth.Enabled =
				activate;
		}

		#endregion

		#region Paint Functions

		private void picInputPattern_Paint(object sender, System.Windows.Forms.PaintEventArgs e) {
			// This function draws the input pattern.
			System.Drawing.Graphics g = e.Graphics;  // fetch drawing handle
			// draw grid
			for (int i = 0; i <= network.iWidth; i++)
				g.DrawLine(Pens.Black, 10 + (picInputPattern.Width - 25) * i / network.iWidth, 10, 10 + (picInputPattern.Width - 25) * i / network.iWidth, picInputPattern.Height - 15);
			for (int j = 0; j <= network.iHeight; j++)
				g.DrawLine(Pens.Black, 10, 10 + (picInputPattern.Height - 25) * j / network.iHeight, picInputPattern.Width - 15, 10 + (picInputPattern.Height - 25) * j / network.iHeight);
			// fill in active squares
			for (int i = 0; i < network.iWidth; i++) {
				for (int j = 0; j < network.iHeight; j++) {
					if (network.bInputPattern[j * network.iWidth + i] == true) {
						int left = 10 + (picInputPattern.Width - 25) * i / network.iWidth + 2;
						int top = 10 + (picInputPattern.Height - 25) * j / network.iHeight + 2;
						int right = 10 + (picInputPattern.Width - 25) * (i + 1) / network.iWidth - 1;
						int bottom = 10 + (picInputPattern.Height - 25) * (j + 1) / network.iHeight - 1;
						g.FillRectangle(Brushes.Black, left, top, right - left, bottom - top);
					}
				}
			}
		}

		private void picOutputPattern_Paint(object sender, System.Windows.Forms.PaintEventArgs e) {
			// This function draws the output pattern.
			System.Drawing.Graphics g = e.Graphics;  // fetch drawing handle
			// draw grid
			for (int i = 0; i <= network.iWidth; i++)
				g.DrawLine(Pens.Black, 10 + (picOutputPattern.Width - 25) * i / network.iWidth, 10, 10 + (picOutputPattern.Width - 25) * i / network.iWidth, picOutputPattern.Height - 15);
			for (int j = 0; j <= network.iHeight; j++)
				g.DrawLine(Pens.Black, 10, 10 + (picOutputPattern.Height - 25) * j / network.iHeight, picOutputPattern.Width - 15, 10 + (picOutputPattern.Height - 25) * j / network.iHeight);
			// fill in active squares
			for (int i = 0; i < network.iWidth; i++) {
				for (int j = 0; j < network.iHeight; j++) {
					if (((Neuron) network.aNeurons[j * network.iWidth + i]).x == true) {
						int left = 10 + (picOutputPattern.Width - 25) * i / network.iWidth + 2;
						int top = 10 + (picOutputPattern.Height - 25) * j / network.iHeight + 2;
						int right = 10 + (picOutputPattern.Width - 25) * (i + 1) / network.iWidth - 1;
						int bottom = 10 + (picOutputPattern.Height - 25) * (j + 1) / network.iHeight - 1;
						g.FillRectangle(Brushes.Black, left, top, right - left, bottom - top);
					}
				}
			}
		}

		private void picNetwork_Paint(object sender, System.Windows.Forms.PaintEventArgs e) {
			// This function draws the neural network diagram.
			int iEllipseSize = (int) Math.Min(picNetwork.Width / 40, picNetwork.Height / 40);
			Font font = new Font("Times New Roman", 7.0f);
			StringFormat format = new StringFormat();
			format.Alignment = StringAlignment.Center;
			System.Drawing.Graphics g = e.Graphics;
			g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
			g.FillRectangle(Brushes.BlanchedAlmond, e.ClipRectangle);

			// find selected neuron
			int currentTime = System.DateTime.Now.Hour * 3600 + System.DateTime.Now.Minute * 60 + System.DateTime.Now.Second;
			if (currentTime - this.iLastClicked > 10) {
				if (currentTime - this.iLastMoved > 1) {
					iLastMoved = currentTime;
					iSelectedNeuron++;
				}
			}
			if (iSelectedNeuron != -1) {  // viewing a particular neuron
				iSelectedNeuron = iSelectedNeuron % (network.iWidth * network.iHeight);
				Neuron selectedNeuron = (Neuron) network.aNeurons[iSelectedNeuron];
				int selectedX = iSelectedNeuron % network.iWidth;
				int selectedY = iSelectedNeuron / network.iWidth;
				int iTotal = 0;
				foreach (object synapseobject in selectedNeuron.aSynapses) {
					Neuron destNeuron = null;
					Synapse s = (Synapse) synapseobject;
					iTotal += s.weight;
					if (s.neurons[0] == selectedNeuron)
						destNeuron = s.neurons[1];
					else if (s.neurons[1] == selectedNeuron)
						destNeuron = s.neurons[0];
					if (destNeuron != null) {
						int index = network.aNeurons.IndexOf(destNeuron);
						// draw neuron
						int color;
						if (network.iTemplates == 0)
							color = 128;
						else
							color = 128 + (int) (127.0 * (Math.Abs(s.weight) / (float) network.iTemplates));
						color = (int) Math.Min(color, 255);
						color = (int) Math.Max(color, 0);
						Brush brush = new SolidBrush(s.weight >= 0 ? Color.FromArgb(color, 0, 0) : Color.FromArgb(0, 0, color));
						int x = index % network.iWidth;
						int y = index / network.iWidth;
						g.FillEllipse(brush, 10 + (int) ((picNetwork.Width - 25) * (x + 0.5) / network.iWidth), 10 + (int) ((picNetwork.Height - 25) * (y + 0.5) / network.iHeight), iEllipseSize, iEllipseSize);
						g.DrawEllipse(Pens.Black, 10 + (int) ((picNetwork.Width - 25) * (x + 0.5) / network.iWidth), 10 + (int) ((picNetwork.Height - 25) * (y + 0.5) / network.iHeight), iEllipseSize, iEllipseSize);
						int drawX = 11 + (int) ((picNetwork.Width - 25) * ((index % network.iWidth) + 0.5) / network.iWidth) + iEllipseSize / 2;
						int drawY = 11 + (int) ((picNetwork.Height - 25) * ((index / network.iWidth) + 0.5) / network.iHeight) - 15;
						g.DrawString(s.weight.ToString(), font, Brushes.Black, drawX, drawY, format);
					}
				}
				int drawSelectedX = 11 + (int) ((picNetwork.Width - 25) * ((iSelectedNeuron % network.iWidth) + 0.5) / network.iWidth) + iEllipseSize / 2;
				int drawSelectedY = 11 + (int) ((picNetwork.Height - 25) * ((iSelectedNeuron / network.iWidth) + 0.5) / network.iHeight) - 15;
				g.DrawString(iTotal.ToString(), font, Brushes.Black, drawSelectedX, drawSelectedY, format);
				Brush brushSelected = new SolidBrush(Color.FromArgb(0, 255, 0));
				g.DrawEllipse(Pens.Black, 10 + (int) ((picNetwork.Width - 25) * (selectedX + 0.5) / network.iWidth), 10 + (int) ((picNetwork.Height - 25) * (selectedY + 0.5) / network.iHeight), iEllipseSize, iEllipseSize);
				g.FillEllipse(brushSelected, 10 + (int) ((picNetwork.Width - 25) * (selectedX + 0.5) / network.iWidth), 10 + (int) ((picNetwork.Height - 25) * (selectedY + 0.5) / network.iHeight), iEllipseSize, iEllipseSize);
			}
			else {  // viewing network as a whole
				// calculate totals
				int[] total = new int[network.aNeurons.Count];
				int max = 0;
				foreach (object o in network.aSynapses) {
					Synapse s = (Synapse) o;
					total[network.aNeurons.IndexOf(s.neurons[0])] += s.weight;
					total[network.aNeurons.IndexOf(s.neurons[1])] += s.weight;
				}

				// find max
				for (int i = 0; i < total.Length; i++)
					max = Math.Max(max, Math.Abs(total[i]));

				// draw network
				for (int index = 0; index < network.aNeurons.Count; index++) {
					int color;
					if (network.iTemplates == 0)
						color = 128;
					else
						color = 128 + (int) (127 * Math.Abs(total[index]) / max);
					color = (int) Math.Min(color, 255);
					color = (int) Math.Max(color, 0);
					Brush brush = new SolidBrush(total[index] >= 0 ? Color.FromArgb(color, 0, 0) : Color.FromArgb(0, 0, color));
					int x = index % network.iWidth;
					int y = index / network.iWidth;
					g.FillEllipse(brush, 10 + (int) ((picNetwork.Width - 25) * (x + 0.5) / network.iWidth), 10 + (int) ((picNetwork.Height - 25) * (y + 0.5) / network.iHeight), iEllipseSize, iEllipseSize);
					g.DrawEllipse(Pens.Black, 10 + (int) ((picNetwork.Width - 25) * (x + 0.5) / network.iWidth), 10 + (int) ((picNetwork.Height - 25) * (y + 0.5) / network.iHeight), iEllipseSize, iEllipseSize);
					int drawX = 11 + (int) ((picNetwork.Width - 25) * ((index % network.iWidth) + 0.5) / network.iWidth) + iEllipseSize / 2;
					int drawY = 11 + (int) ((picNetwork.Height - 25) * ((index / network.iWidth) + 0.5) / network.iHeight) - 15;
					g.DrawString(total[index].ToString(), font, Brushes.Black, drawX, drawY, format);
				}
			}
		}

		#endregion

		#region Load/Save Functions

		private void btnSaveNetwork_Click(object sender, System.EventArgs e) {
			SaveFileDialog s = new SaveFileDialog();
			s.Filter = "Network Files (*.net)|*.net";
			s.OverwritePrompt = true;
			if (s.ShowDialog() == DialogResult.OK) {
				XmlTextWriter save = new XmlTextWriter(s.FileName, System.Text.Encoding.UTF8);
				save.Formatting = Formatting.Indented;
				save.IndentChar = ' ';
				save.WriteStartDocument();
				save.WriteStartElement("HopfieldNetwork");
				save.Indentation++;
				// write parameters
				save.WriteStartElement("Parameters");
				save.Indentation++;
				save.WriteElementString("Name", network.strName);
				save.WriteElementString("Templates", network.iTemplates.ToString());
				save.WriteElementString("Width", network.iWidth.ToString());
				save.WriteElementString("Height", network.iHeight.ToString());
				save.Indentation--;
				save.WriteEndElement();
				// write neuron data
				save.WriteStartElement("Synapses");
				save.Indentation++;
				foreach (Object synapseobject in network.aSynapses)
					save.WriteElementString("Weight", ((Synapse) synapseobject).weight.ToString());
				save.Indentation--;
				save.WriteEndElement();  // close Synapses
				// close XML file
				save.Indentation--;
				save.WriteEndElement();
				save.WriteEndDocument();
				save.Close();
			}
		}

		private void btnLoadNetwork_Click(object sender, System.EventArgs e) {
			XmlTextReader xtr = null;
			string name = "";
			int width = 0;
			int height = 0;
			int templates = 0;
			HopfieldNetwork network = null;
			OpenFileDialog open = new OpenFileDialog();
			open.Filter = "Network files (*.net)|*.net";
			if (open.ShowDialog() == DialogResult.OK) {  // selected an XML file
				try {
					xtr = new XmlTextReader(open.FileName);
					XmlValidatingReader xvr = new XmlValidatingReader(xtr);
					xvr.ValidationType = ValidationType.Schema;
					xvr.ValidationEventHandler += new ValidationEventHandler(ValidationErrors);
					XmlSchemaCollection xsc = new XmlSchemaCollection();
					xsc.Add("", System.Reflection.Assembly.GetExecutingAssembly().Location.Substring(0, System.Reflection.Assembly.GetExecutingAssembly().Location.Length - 20) + @"Schema Files\Network.xsd");
					xvr.Schemas.Add(xsc);
					XmlDocument xd = new XmlDocument();
					XmlLoadErrors = "";
					xd.Load(xvr);
					if (XmlLoadErrors.Length > 0)
						MessageBox.Show("Errors loading network document. This does not appear to be a valid network file.", "Errors Loading Network Document", MessageBoxButtons.OK, MessageBoxIcon.Error);
					else {
						foreach (XmlNode x1 in xd.ChildNodes) {
							if (x1.Name == "HopfieldNetwork") {
								foreach (XmlNode x2 in x1.ChildNodes) {
									if (x2.Name == "Parameters") {
										foreach (XmlNode x3 in x2.ChildNodes) {
											if (x3.Name == "Name")
												name = x3.InnerText;
											else if (x3.Name == "Templates")
												templates = (int) Int32.Parse(x3.InnerText);
											else if (x3.Name == "Width")
												width = (int) Int32.Parse(x3.InnerText);
											else if (x3.Name == "Height") {  // at this point, create new network.
												height = (int) Int32.Parse(x3.InnerText);
												network = new HopfieldNetwork(width, height);
												network.strName = name;
												network.iTemplates = templates;
											}
										}
									}
									else if ((x2.Name == "Synapses") && (network != null)) {
										int iCount = 0;
										foreach (XmlNode x3 in x2.ChildNodes) {
											if (x3.Name == "Weight")
												((Synapse) network.aSynapses[iCount++]).weight = Int32.Parse(x3.InnerText);
										}
									}
								}
							}
						}
						// update controls
						if (network != null) {
							txtName.Text = name;
							txtTemplates.Text = templates.ToString();
							nudNetworkWidth.Value = width;
							nudNetworkHeight.Value = height;
							this.network = network;
						}
						picNetwork.Refresh();
					}
				} catch (Exception ex) {
					MessageBox.Show(ex.Message);
				}
				finally {
					if (xtr != null)
						xtr.Close();
				}
			}
		}

		private void btnSavePattern_Click(object sender, System.EventArgs e) {
			SaveFileDialog s = new SaveFileDialog();
			s.Filter = "Pattern Files (*.pat)|*.pat";
			s.OverwritePrompt = true;
			if (s.ShowDialog() == DialogResult.OK) {
				XmlTextWriter save = new XmlTextWriter(s.FileName, System.Text.Encoding.UTF8);
				save.Formatting = Formatting.Indented;
				save.IndentChar = ' ';
				save.WriteStartDocument();
				save.WriteStartElement("HopfieldNetworkPattern");
				save.Indentation++;
				// write parameters
				save.WriteStartElement("Size");
				save.Indentation++;
				save.WriteElementString("Width", network.iWidth.ToString());
				save.WriteElementString("Height", network.iHeight.ToString());
				save.Indentation--;
				save.WriteEndElement();
				// write neuron data
				save.WriteStartElement("Values");
				save.Indentation++;
				for (int i = 0; i < network.bInputPattern.Length; i++)
					save.WriteElementString("Pixel", network.bInputPattern[i].ToString());
				save.Indentation--;
				save.WriteEndElement();  // close Values
				// close XML file
				save.Indentation--;
				save.WriteEndElement();
				save.WriteEndDocument();
				save.Close();
			}
		}

		private void btnLoadPattern_Click(object sender, System.EventArgs e) {
			XmlTextReader xtr = null;
			OpenFileDialog open = new OpenFileDialog();
			open.Filter = "Pattern files (*.pat)|*.pat";
			if (open.ShowDialog() == DialogResult.OK) {  // selected an XML file
				try {
					xtr = new XmlTextReader(open.FileName);
					XmlValidatingReader xvr = new XmlValidatingReader(xtr);
					xvr.ValidationType = ValidationType.Schema;
					xvr.ValidationEventHandler += new ValidationEventHandler(ValidationErrors);
					XmlSchemaCollection xsc = new XmlSchemaCollection();
					xsc.Add("", System.Reflection.Assembly.GetExecutingAssembly().Location.Substring(0, System.Reflection.Assembly.GetExecutingAssembly().Location.Length - 20) + @"Schema Files\Pattern.xsd");
					xvr.Schemas.Add(xsc);
					XmlDocument xd = new XmlDocument();
					XmlLoadErrors = "";
					xd.Load(xvr);
					if (XmlLoadErrors.Length > 0)
						MessageBox.Show("Errors loading pattern document. This does not appear to be a valid pattern file.", "Errors Loading Pattern Document", MessageBoxButtons.OK, MessageBoxIcon.Error);
					else {
						foreach (XmlNode x1 in xd.ChildNodes) {
							if (x1.Name == "HopfieldNetworkPattern") {
								foreach (XmlNode x2 in x1.ChildNodes) {
									if (x2.Name == "Size") {
										foreach (XmlNode x3 in x2.ChildNodes) {
											if (x3.Name == "Width") {
												if ((int) Int32.Parse(x3.InnerText) != network.iWidth)
													throw new Exception("Pattern width does not match current network.");
											}
											else if (x3.Name == "Height") {
												if ((int) Int32.Parse(x3.InnerText) != network.iHeight)
													throw new Exception("Pattern height does not match current network.");
											}
										}
									}
									else if ((x2.Name == "Values") && (network != null)) {
										int iCount = 0;
										foreach (XmlNode x3 in x2.ChildNodes) {
											if (x3.Name == "Pixel")
												network.bInputPattern[iCount++] = Boolean.Parse(x3.InnerText);
										}
									}
								}
							}
						}
						// update controls
						this.picInputPattern.Refresh();
					}
				} catch (Exception ex) {
					MessageBox.Show(ex.Message);
				}
				finally {
					if (xtr != null)
						xtr.Close();
				}
			}
		}

		public void ValidationErrors(object sender, ValidationEventArgs e) {
			XmlLoadErrors += e.Message + "\r\n";
		}
		#endregion

	}
}
